/*
 * Created on Dec 10, 2004
 *
 */
package AccordionSequenceDrawer;

import AccordionDrawer.SplitCell;
import AccordionDrawer.SplitLine;
import java.util.ArrayList;
import java.util.Iterator;

import java.util.TreeSet;
import java.awt.Color;

/**
 * @author jslack
 * 
 * ColumnColorList: tree of colors for sequential sequences that share the same color value
 * 
 * Each vertical splitCell (split cells that belong to splitLine[X] have a horizontal offset and can be seen
 * as a representative of a column, hence vertical), has a culling object for its range.  The culling
 * object is created on demand when the splitCell reaches the size of a block (or smaller) in the rendering
 * process (the size of each block required for a rendering is computed before a scene is drawn by
 * asd.preDrawNewScene when it computes the listOfSites).  If the cullingObject of a splitCell in listOfSites
 * is a columnColorList, then we have previously computed the colors for sequence blocks in listOfSequences
 * and can use the columnColors tree to retrieve colors for any range of sequences for this splitCell.
 * 
 * When we need to compute the ColumnColorList for a splitCell, we create the ColumnColorList for its
 * children (this process is recursive, so if the children are already ColumnColorLists we don't need to
 * recompute, both children are null causes the base case where we create a new ColumnNumberList for that
 * splitCell) which creates the columnNumberList for their parent splitCell as the cullingObject.  After the children
 * are created, the columnNumberList (an array of integers that for each sequence for this range of sites)
 * is tallied and we determine colors for each sequence for each range of sites.  The colors that represent
 * each sequence are combined into ranges (a ColumnColorSet) and added to the columnColors treeset.  The
 * columnNumberList is added to the parent of this splitCell's cullingObject (which is either null or a
 * columnNumberList).
 *
 */
public class ColumnColorList {

	TreeSet columnColors;
	
//	 GapsACGTUN, add 1 to most popular letter in a column, never reset, use to break ties
	static ColumnNumberSet globalCount = new ColumnNumberSet(null, ' '); 
	
	public ColumnColorList(SplitCell sc, SplitLine sl, ArrayList sequences)
	{
		if (sc.cullingObject instanceof ColumnColorList)
		{
			System.out.println("Error: culling object should be null or columnnumberlist when calling column color list constructor");
			return;
		}
		SplitCell parent = sl.getSplitCell(sc.parent);
		if (sc.min+1 >= sc.max) // this is impossible
		{
			System.out.println("Badly formed split cell with adjacent min and max");
		}
		else
		{
			if (sl == null || sc == null || sc.kid == null)
				System.out.println("either sl or sc or sc.kid is null: " + sl + " " + sc + " " + sc.kid);
			SplitCell leftKid = sl.getSplitCell(sc.kid[0]);
			SplitCell rightKid = sl.getSplitCell(sc.kid[1]);
			if (leftKid == null && rightKid == null)
			// base case, 2 or 1 sitenodes separated by sc (sc.min+1 through sc.max)
				sc.cullingObject = new ColumnNumberList(sequences, sc.min+1, sc.max);
			if (leftKid != null && !(leftKid.cullingObject instanceof ColumnColorList))
				// left recursion
				new ColumnColorList(leftKid, sl, sequences); // leftKid.cullobject gets set internally
			if (rightKid != null && !(rightKid.cullingObject instanceof ColumnColorList))
				// right recursion
				new ColumnColorList(rightKid, sl, sequences); // rightKid.cullobject gets set internally
			if (!(sc.cullingObject instanceof ColumnNumberList))
			{
				System.out.println("Error: constructing kids as column color lists should make the parent a number list");
				return;
			}
			// now safe to use the other constructor
			sc.cullingObject = new ColumnColorList(sc, sl, sequences, true);
		}
	}
	
	private ColumnColorList(SplitCell sc, SplitLine sl, ArrayList sequences, boolean dummy)
	{
		SplitCell leftKid = sl.getSplitCell(sc.kid[0]);
		SplitCell rightKid = sl.getSplitCell(sc.kid[1]);
		if (sc.cullingObject instanceof ColumnNumberList &&
				(leftKid == null || leftKid.cullingObject instanceof ColumnColorList) &&
				(rightKid == null || rightKid.cullingObject instanceof ColumnColorList))
		{
			if (leftKid != null && rightKid == null && leftKid.max+1 == sc.max)
			{
				sc.cullingObject = new ColumnNumberList((ColumnNumberList)sc.cullingObject, new ColumnNumberList(sequences, sc.max, sc.max));
			}
			columnColors = new TreeSet();
			ColumnNumberList cnl = (ColumnNumberList)sc.cullingObject;
			
			int start = 0;
			ColumnNumberSet currCns = cnl.get(start);
			char currColor;
			char prevColor = currCns.getColor();
			ColumnNumberSet columnCNS = new ColumnNumberSet(null, prevColor); // for global count

			for (int i = 1; i < cnl.columnNumberList.size(); i++)
			{
				currCns = cnl.get(i);
				currColor = currCns.getColor();
				columnCNS.addColor(currColor); // global
				if (currColor != prevColor)
				{
					// even blanks get added, find first non blank when drawing set of sequences
					columnColors.add(new ColumnColorSet(prevColor, start, i-1));
					start = i;
				}
				prevColor = currColor;
			}
			columnColors.add(new ColumnColorSet(prevColor, start, cnl.columnNumberList.size()-1));
			globalCount.addColor(columnCNS.getColor());
//			System.out.println("global: " + globalCount);
			SplitCell parent = sl.getSplitCell(sc.parent);
			if (parent != null)
				parent.cullingObject = new ColumnNumberList((ColumnNumberList)parent.cullingObject, cnl);
		}
		else
		{
			System.out.println("Something is the wrong type in the private constructor for column color list");
		}
	}
	
	// min, max are sequence numbers
	// color is a site node color
	public void addToColors(int min, int max, char color)
	{
		columnColors.add(new ColumnColorSet(color, min, max));
	}
	
	// get some color that is in the range [min,max]
	// min and max are sequence numbers, color is a site node color
	public Color getColor(int min, int max)
	{
		char returnColor = '-';
		ColumnColorSet fakeSet = new ColumnColorSet('-', min, max);
		ColumnColorSet.getColor = true;
		if (columnColors.contains(fakeSet))
		{
			returnColor = ColumnColorSet.returnColorSet.colorLetter;
			ColumnColorSet.returnColorSet = null; // reset
		}
		return Sequence.getColor(returnColor);
	}

	public String toString()
	{
		Iterator iter = columnColors.iterator();
		String str = "[";
		while (iter.hasNext())
		{
			ColumnColorSet ccs = (ColumnColorSet)iter.next();
			if (iter.hasNext())
				str += ccs + ", ";
			else
				str += ccs + "]";
		}
		return str;
	}
}
